<?php
namespace WapplerSystems\Cleverreach\Form\Validator;

/*
 * This file is part of the TYPO3 CMS project.
 *
 * It is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License, either version 2
 * of the License, or any later version.
 *
 * For the full copyright and license information, please read the
 * LICENSE.txt file that was distributed with this source code.
 *
 * The TYPO3 project - inspiring people to share!
 */

use TYPO3\CMS\Core\Utility\GeneralUtility;
use TYPO3\CMS\Extbase\Object\ObjectManager;
use WapplerSystems\Cleverreach\Service\ConfigurationService;
use TYPO3\CMS\Extbase\Validation\Validator\AbstractValidator;

/**
 * Mx Validator for email addresses
 *
 * @api
 */
class MxValidator extends AbstractValidator
{

    /**
     * @var \WapplerSystems\Cleverreach\CleverReach\Api
     * @inject
     */
    protected $api;


    /**
     * Checks if the given value is already in the list
     *
     * @param mixed $value The value that should be validated
     * @api
     */
    public function isValid($email)
    {

        // Early return in case input is not a string
        if (!is_string($email)) {
            return;
        }
        $atPosition = strrpos($email, '@');
        if (!$atPosition || $atPosition + 1 === strlen($email)) {
            // Return if no @ found or it is placed at the very beginning or end of the email
            $this->addError(
                $this->translateErrorMessage(
                    'validator.mx.failed',
                    'cleverreach'
                ),
                1546044894
            );
            return;
        }
        $domain = substr($email, $atPosition + 1);
        if (!preg_match('/^[a-z0-9.\\-]*$/i', $domain)) {
            try {
                $domain = GeneralUtility::idnaEncode($domain);
            } catch (\InvalidArgumentException $exception) {
                $this->addError(
                    $this->translateErrorMessage(
                        'validator.mx.failed',
                        'cleverreach'
                    ),
                    1546044894
                );
                return;
            }
        }

        if (!checkdnsrr($domain, 'MX')) {
            $this->addError(
                $this->translateErrorMessage(
                    'validator.mx.failed',
                    'cleverreach'
                ),
                1546044893
            );
        }
    }

}
